(***********************************************************************)
(*                                                                     *)
(*                              Ref-indic                              *)
(*                                                                     *)
(*                        Refinement indicators                        *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2013-2014 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: main.ml,v 1.4 2014-03-14 15:47:26 fclement Exp $ *)

module Nat = Unatural;;
module Cmd_opt = Ucommand_option;;

let pf = Pio.foreman_make_printing_facilities ();;

let piob = Uunix.std_process_io_buffers;;

let wrap tw = Pio.worker_test_wrap_task piob tw;;

let make_input make_wrapper fun_name =
  let { Pio.tname = tname; inj = inj; _; } = make_wrapper pf fun_name in
  wrap tname inj
;;

let make_init_input = make_input Wrapper.make_init_wrapper "init"
and make_grad_input = make_input Wrapper.make_grad_wrapper "grad"
and make_optim_input = make_input Wrapper.make_optim_wrapper "optim"
and make_final_input = make_input Wrapper.make_final_wrapper "final"
;;

(* Command-line options. *)

let init_dim = Nat._0_;;

let { Uaref.get = get_dim; set = set_dim; } =
  Uaref.make_aref_unique "dimension" init_dim
;;

let option_dim =
  {
   Cmd_opt.key = "-dim";
   spec = Arg.Int (fun i -> set_dim (Nat.of_int i));
   doc = Printf.sprintf "<nat>  Set dimension to <nat>.";
  }
;;

let user_add x = Cmd_opt.user_add (Cmd_opt.of_regular x);;

let add_options () = user_add option_dim;;

add_options ();;

Cmd_opt.set_usage_msg
  (Format.sprintf
     "@.%s -- Generate identity worker input@.@.\
      Usage: %s [<option>...] -dim <nat>@.@.\
      Options are:@."
     Sys.argv.(0) Sys.argv.(0))
;;

(* Main. *)

let main () =
  Cmd_opt.parse ();
  
  let dim = get_dim () in
  
  let array_make x = Uarray.make dim x in
  let len = Nat._9_ in
  let matrix_make x = Umatrix.make dim len x in
  let matrix_init f = Umatrix.init dim len f in
  
  let constant_matrix = matrix_make in
  let two_zone_matrix j1 x0 x1 =
    matrix_init
      (fun _i j -> if j < j1 then x0 else x1) in
  
  make_init_input dim;
  
  let pm = constant_matrix 127. in
  make_grad_input pm;
  
  let ppm = two_zone_matrix 5 Nat._0_ Nat._1_
  and maa = array_make [| 0.; 0.; |] in
  make_optim_input (ppm, maa);
  
  let pm = two_zone_matrix 5 0. 127. in
  make_final_input pm;
;;

main ();;
