(***********************************************************************)
(*                                                                     *)
(*                              Ref-indic                              *)
(*                                                                     *)
(*                        Refinement indicators                        *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2013-2014 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: worker.mli,v 1.4 2014-03-08 16:55:01 fclement Exp $ *)

(** {3 Generic part of workers for the inversion platform} *)

open Wire_types;;

(** {6 Printing facilities} *)

type 'a printer = 'a Format_printer.printer;;

type 'a string_printer = string -> 'a -> string;;
type ('a, 'b) string_printer_lifter = 'a printer -> 'b string_printer;;

val format_sprint : ('a, 'a) string_printer_lifter;;

val sprint_nat : Unatural.t string_printer;;
val sprint_float : float string_printer;;

val sprint_homogeneous_couple : ('a, 'a * 'a) string_printer_lifter;;
val sprint_array : ('a, 'a array) string_printer_lifter;;
val sprint_array_array : ('a, 'a array array) string_printer_lifter;;
val sprint_matrix : ('a, 'a Umatrix.t) string_printer_lifter;;

val sprint_nat_couple : (Unatural.t * Unatural.t) string_printer;;
val sprint_nat_matrix : Unatural.t Umatrix.t string_printer;;

val sprint_float_array_array : float array array string_printer;;
val sprint_float_matrix : float Umatrix.t string_printer;;

val um_invalid_arg : Upervasives.function_name -> Upervasives.message -> 'a;;
val um_failwith : Upervasives.function_name -> Upervasives.message -> 'a;;

(** {6 Tasks} *)

val coarse_parameter_to_fine_parameter :
  partition -> coarse_parameter -> fine_parameter
;;

val check_valid_dimension :
  Upervasives.function_name -> dimension -> dimension -> unit
;;

val make_init :
  (Pio.spf -> dimension -> size * partition * coarse_parameter) ->
  (Pio.spf -> partition * coarse_parameter -> coarse_parameter * cost) ->
  Pio.spf -> dimension -> size * partition * coarse_parameter * cost
;;

val make_grad :
  (Pio.spf -> fine_parameter -> fine_gradient) ->
  Pio.spf -> fine_parameter -> fine_gradient
;;

val make_optim :
  (Pio.spf -> partition -> coarse_parameter -> coarse_parameter * cost) ->
  Pio.spf -> partition * coarse_parameter -> coarse_parameter * cost
;;

val make_final :
  (Pio.spf -> fine_parameter -> unit) ->
  Pio.spf -> fine_parameter -> unit
;;

(** {6 Main loop} *)

val main_loop :
  Pio.welcome_action -> Pio.farewell_action ->
  (dimension, size * partition * coarse_parameter * cost) Pio.task ->
  (fine_parameter, fine_gradient) Pio.task ->
  (partition * coarse_parameter, coarse_parameter * cost) Pio.task ->
  (fine_parameter, unit) Pio.task ->
  unit
;;
