CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                     C
C                              Ref-indic                              C
C                                                                     C
C                        Refinement indicators                        C
C                                                                     C
C               EPI Pomdapi, INRIA Paris - Rocquencourt               C
C                                                                     C
C  Copyright 2013-2014 INRIA.                                         C
C  Institut National de Recherche en Informatique et en Automatique.  C
C  All rights reserved. This file is distributed only by permission.  C
C                                                                     C
C  Hend Ben Ameur <hbenameur@yahoo.ca>                                C
C  Francois Clement <Francois.Clement@inria.fr>                       C
C  Pierre Weis <Pierre.Weis@inria.fr>                                 C
C                                                                     C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

C     $Id: identity_model.fpp,v 1.8 2014-03-14 20:37:27 fclement Exp $

#include "identity_model_constants.h"

      subroutine load_ascii_pnm(s_len, sbr)
C     Load ASCII PNM image (only accepts a subset of ASCII PNM images).
      
      implicit none
      
      include 'pio_user_error_definition.fh'
      include 'worker_init_definition.fh'
      include 'identity_model_definition.fh'
      
C     Input arguments.
      character sbr*(*)
      integer s_len
      
C     Local variables.
      character comment*1
      integer ix, iy, i, j, k
      FLOAT maxval, line(3 * MAX_WIDTH)
      
C     External function.
      logical pio_user_get_debug_mode
      
      read(PIO_USER_IOUNIT, *) comment
      
      if (comment .ne. '#') then
         call error_wrong_pnm_format(s_len, sbr)
         return
      endif
      
      read(PIO_USER_IOUNIT, *) size_x, size_y
      if (size_x .gt. MAX_WIDTH) then
         call pio_user_error_too_large_length
     +        (s_len, sbr, 5, 'width', MAX_WIDTH, size_x)
         return
      endif
      
      read(PIO_USER_IOUNIT, *) maxval
      
      i = 1
      do iy = 1, size_y
         read(PIO_USER_IOUNIT, *) (line(k), k = 1, dim*size_x)
         k = 1
         do ix = 1, size_x
            do j = 1, dim
               d(i, j) = line(k) * MAX_BYTE_FLOAT / maxval
               k = k + 1
            enddo
            i = i + 1
         enddo
      enddo
      
      if (pio_user_get_debug_mode()) then
         do j = 1, dim
            do i = 1, size_x * size_y
               write(PIO_USER_DEBUG, 1001) i, j, d(i, j)
            enddo
         enddo
      endif
      
 1001 format('i = ', i2, ', j = ', i2, ', d = ', DEBUG_FLOAT_FMT)
      
      return
      end


      subroutine load_pnm_image(s_len, sbr)
C     Load PNM image (only accepts a subset of PNM images).
      
      implicit none
      
      include 'pio_user_error_definition.fh'
      include 'worker_init_definition.fh'
      include 'identity_model_definition.fh'
      
C     Input arguments.
      character sbr*(*)
      integer s_len
      
C     Local variables.
      character magic*2
      
      open(unit=PIO_USER_IOUNIT, file=fname_in(1:fni_len),
     +     status='old', access='sequential', form='formatted',
     +     iostat=ier)
      if (ier .ne. 0) then
         call pio_user_error_open_failure
     +        (s_len, sbr, fni_len, fname_in)
         return
      endif
      
      read(PIO_USER_IOUNIT, *) magic
      
      if ((magic .eq. 'P1') .or. (magic .eq. 'P4') .or.
     +     (magic .eq. 'P2') .or. (magic .eq. 'P5')) then
         dim = 1
      else if ((magic .eq. 'P3') .or. (magic .eq. 'P6')) then
         dim = 3
      else if (magic .eq. 'P7') then
         dim = 4
      else
         call error_unbound_magic_number(s_len, sbr, magic)
         return
      endif
      
      if ((magic .eq. 'P1') .or.
     +     (magic .eq. 'P2') .or.
     +     (magic .eq. 'P3')) then
         call load_ascii_pnm(s_len, sbr)
         if (ier .ne. 0) return
      else if ((magic .eq. 'P4') .or.
     +     (magic .eq. 'P5') .or.
     +     (magic .eq. 'P6')) then
         call error_not_yet_implemented(s_len, sbr, 12, 'load raw PNM')
         return
      else if (magic .eq. 'P7') then
         call error_not_yet_implemented(s_len, sbr, 8, 'load PAM')
         return
      endif
      
      close(PIO_USER_IOUNIT)
      
      return
      end


      subroutine save_ascii_pnm(p)
C     Save ASCII PGM or PPM image.
      
      implicit none
      
      include 'pio_user_error_definition.fh'
      include 'worker_init_definition.fh'
      
C     Input argument.
      FLOAT p(WORKER_MAX_SIZE, WORKER_MAX_DIMENSION)
      
C     Local variables.
      integer maxval, ix, iy, iy1, i, j, k, line(3 * MAX_WIDTH)
      
      write(PIO_USER_IOUNIT, '(a)')
     +     '# Generated by Ref-indic Fortran identity model.'
      
      write(PIO_USER_IOUNIT, *) size_x, size_y
      
      maxval = MAX_BYTE_INT
      write(PIO_USER_IOUNIT, *) maxval
      
      iy1 = 1
      do iy = 1, size_y
         i = iy1
         k = 1
         do ix = 1, size_x
            do j = 1, dim
               line(k) = INT_OF_FLOAT(p(i, j))
            enddo
            i = i + 1
            k = k + 1
         enddo
         write(PIO_USER_IOUNIT, *) (line(k), k = 1, dim*size_x)
         iy1 = iy1 + size_x
      enddo
      
      return
      end


      subroutine save_pnm_image(s_len, sbr, p, fn_len, fname)
C     Save PNM image.
      
      implicit none
      
      include 'pio_user_error_definition.fh'
      include 'worker_init_definition.fh'
      
C     Input arguments.
      character sbr*(*), fname*(*)
      integer s_len, fn_len
      FLOAT p(WORKER_MAX_SIZE, WORKER_MAX_DIMENSION)
      
C     Local variables.
      character magic*2
      
      open(unit=PIO_USER_IOUNIT, file=fname(1:fn_len),
     +     status='unknown', access='sequential', form='formatted',
     +     iostat=ier)
      if (ier .ne. 0) then
         call pio_user_error_open_failure(s_len, sbr, fn_len, fname)
         return
      endif
      
      if (dim .eq. 1) then
         magic = 'P2'
      else if (dim .eq. 3) then
         magic = 'P3'
      else
         call error_unbound_dimension(s_len, sbr, dim)
         return
      endif
      write(PIO_USER_IOUNIT, '(a2)') magic
      
      call save_ascii_pnm(p)
      if (ier .ne. 0) return
      
      close(PIO_USER_IOUNIT)
      
      return
      end


      subroutine parse_config_file
C     Parse the configuration file.
      
      implicit none
      
      include 'pio_user_error_definition.fh'
      include 'identity_model_definition.fh'
      
C     Local variables.
      character sbr*17, options_fname*21
      integer s_len, of_len
      
C     External function.
      logical pio_user_get_debug_mode
      
      s_len = 17
      sbr = 'parse_config_file'
      
      of_len = 21
      options_fname = 'identity_model_config'
      
      open(unit=PIO_USER_IOUNIT, file=options_fname, status='old',
     +     access='sequential', form='formatted', iostat=ier)
      if (ier .ne. 0) then
         call pio_user_error_open_failure
     +        (s_len, sbr, of_len, options_fname)
         return
      endif
      
      read(PIO_USER_IOUNIT, *) fni_len
      read(PIO_USER_IOUNIT, *) fname_in(1:fni_len)
      
      if (fni_len .gt. WORKER_MAX_FILE_NAME_LENGTH) then
         call worker_error_initialization
     +        (s_len, sbr, 27, 'input file name is too long')
         return
      endif
      
      if (pio_user_get_debug_mode()) then
         write(PIO_USER_DEBUG, *)
     +        'input file name = ''', fname_in(1:fni_len), ''''
      endif
      
      read(PIO_USER_IOUNIT, *) fno_len
      read(PIO_USER_IOUNIT, *) fname_out(1:fno_len)
      
      if (fno_len .ge. WORKER_MAX_FILE_NAME_LENGTH) then
         call worker_error_initialization
     +        (s_len, sbr, 28, 'output file name is too long')
         return
      endif
      
      if (pio_user_get_debug_mode()) then
         write(PIO_USER_DEBUG, *)
     +        'output file name = ''', fname_out(1:fno_len), ''''
      endif
      
      close(PIO_USER_IOUNIT)
      
      return
      end


      subroutine initialization(pp1, lens1, m1)
C     Initialization.
      
      implicit none
      
      include 'pio_user_error_definition.fh'
      include 'worker_init_definition.fh'
      include 'identity_model_definition.fh'
      
C     Output arguments.
      integer pp1(WORKER_MAX_SIZE, WORKER_MAX_DIMENSION)
      integer lens1(WORKER_MAX_DIMENSION)
      FLOAT m1(WORKER_MAX_ZONE_NUMBER)
      
C     Local variables.
      character sbr*14
      integer s_len, i, j
      
C     External function.
      logical pio_user_get_debug_mode
      
      s_len = 14
      sbr = 'initialization'
      
      call parse_config_file
      if (ier .ne. 0) return
      call load_pnm_image(s_len, sbr)
      if (ier .ne. 0) return
      
      zone_number = dim
      
      do j = 1, dim
         do i = 1, size
            pp1(i, j) = 0
         enddo
         lens1(j) = 1
         m1(j) = PIO_USER_ZERO
      enddo
      
      if (pio_user_get_debug_mode()) then
         do j = 1, dim
            do i = 1, size
               write(PIO_USER_DEBUG, 1001) i, j, pp1(i, j)
            enddo
         enddo
      endif
      
 1001 format('i = ', i2, ', j = ', i2, ', pp1 = ', DEBUG_FLOAT_FMT)
      
      return
      end
      
      
      subroutine gradient_computation(p, g)
C     Gradient computation.
      
      implicit none
      
      include 'pio_user_error_definition.fh'
      include 'worker_init_definition.fh'
      include 'identity_model_definition.fh'
      
C     Input argument.
      FLOAT p(WORKER_MAX_SIZE, WORKER_MAX_DIMENSION)
      
C     Output argument.
      FLOAT g(WORKER_MAX_SIZE, WORKER_MAX_DIMENSION)
      
C     Local variables.
      character sbr*20
      integer s_len, i, j
      
C     External function.
      logical pio_user_get_debug_mode
      
      s_len = 20
      sbr = 'gradient_computation'
      
      do j = 1, dim
         do i = 1, size
            g(i, j) = p(i, j) - d(i, j)
         enddo
      enddo
      
      if (pio_user_get_debug_mode()) then
         do j = 1, dim
            do i = 1, size
               write(PIO_USER_DEBUG, 1001)
     +              i, j, p(i, j), d(i, j), g(i, j)
            enddo
         enddo
      endif
      
 1001 format('i = ', i2, ', j = ', i2,
     +     ', p = ', DEBUG_FLOAT_FMT,
     +     ', d = ', DEBUG_FLOAT_FMT,
     +     ', g = ', DEBUG_FLOAT_FMT)
      
      return
      end
      
      
      subroutine optimization(pp, lens, m, cost)
C     Optimal coarse parameter computation.
      
      implicit none
      
      include 'pio_user_error_definition.fh'
      include 'worker_init_definition.fh'
      include 'identity_model_definition.fh'
      
C     Input arguments.
      integer pp(WORKER_MAX_SIZE, WORKER_MAX_DIMENSION)
      integer lens(WORKER_MAX_DIMENSION)
      
C     Input/output argument.
      FLOAT m(WORKER_MAX_ZONE_NUMBER)
      
C     Output argument.
      FLOAT cost
      
C     Local variables.
      character sbr*36
      integer s_len, count(WORKER_MAX_ZONE_NUMBER), i, j, k, k0, k1
      FLOAT p_ij
      
C     External function.
      logical pio_user_get_debug_mode
      
      s_len = 36
      sbr = 'optimal_coarse_parameter_computation'
      
      do k = 1, zone_number
         m(k) = PIO_USER_ZERO
         count(k) = 0
      enddo
      
C     Partition pp gives zone numbers from 0
C     and components of coarse parameter go from 1.
C     Hence, k0 is initialized to 1, and not 0.
      k0 = 1
      do j = 1, dim
         do i = 1, size
            k = k0 + pp(i, j)
            m(k) = m(k) + d(i, j)
            count(k) = count(k) + 1
         enddo
         k0 = k0 + lens(j)
      enddo
      
      if (pio_user_get_debug_mode()) then
         k = 1
         do j = 1, dim
            do k1 = 1, lens(j)
               write(PIO_USER_DEBUG, 1001) k1, j, m(k), count(k)
               k = k + 1
            enddo
            k0 = k0 + lens(j)
         enddo
      endif
      
      do k = 1, zone_number
         m(k) = m(k) / PIO_FLOAT_OF_INT(count(k))
      enddo
      
      if (pio_user_get_debug_mode()) then
         k = 1
         do j = 1, dim
            do k1 = 1, lens(j)
               write(PIO_USER_DEBUG, 1002) k1, j, m(k)
               k = k + 1
            enddo
            k0 = k0 + lens(j)
         enddo
      endif
      
      k0 = 1
      cost = PIO_USER_ZERO
      do j = 1, dim
         do i = 1, size
            k = k0 + pp(i, j)
            p_ij = m(k)
            cost = cost + (p_ij - d(i, j)) ** 2
         enddo
         k0 = k0 + lens(j)
      enddo
      cost = PIO_USER_HALF * cost
      
      if (pio_user_get_debug_mode()) then
         write(PIO_USER_DEBUG, 1003) cost
      endif
      
 1001 format('k1 = ', i2, ', j = ', i2,
     +     ', m = ', DEBUG_FLOAT_FMT, ', count = ', i2)
 1002 format('k1 = ', i2, ', j = ', i2, ', m = ', DEBUG_FLOAT_FMT)
 1003 format('cost = ', DEBUG_FLOAT_FMT)
      
      return
      end
      
      
      subroutine finalization(p)
C     Finalization.
      
      implicit none
      
      include 'pio_user_error_definition.fh'
      include 'worker_init_definition.fh'
      include 'identity_model_definition.fh'
      
C     Input argument.
      FLOAT p(WORKER_MAX_SIZE, WORKER_MAX_DIMENSION)
      
C     Local variables.
      character sbr*12
      integer s_len
      
      s_len = 12
      sbr = 'finalization'
      
      call save_pnm_image(s_len, sbr, p, fno_len, fname_out)
      if (ier .ne. 0) return
      
      return
      end
